﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Primeval
{
    enum AtomType
    {
        SPEED, HEALTH, AGILITY, SMALLNESS,
        ARMOR, GUN, LASER, ROCKET,

        COLOR, AGGRO, JINK, FREQ, AMP
    }

    class AtomValue
    {
        public int value;

        public static implicit operator int(AtomValue av)
        {
            return av.value;
        }

        public static implicit operator AtomValue(int val)
        {
            return new AtomValue(val);
        }

        public AtomValue()
        {
            value = Game1.random.Next(100) + 2;
        }

        public AtomValue(int val)
        {
            value = val;
        }
    }

    /// <summary>
    /// Describes one "attribute" in a genome (a speed, a gun, an armor plate)
    /// </summary>
    struct Atom
    {
        private const int atomCount = 8;
        private const int mandatoryAtomCount = 5;

        public AtomType type;

        public Dictionary<string, AtomValue> values;

        public const float mutationRate = 0.1f;

        /// <summary>
        /// Fill this atom
        /// </summary>
        public Atom Spawn()
        {
            values = new Dictionary<string, AtomValue>();
            type = (AtomType)Game1.random.Next(atomCount);
            FillValues();
            return this;
        }

        /// <summary>
        /// Fill this atom, forcing a certain type
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public Atom Spawn(AtomType type)
        {
            values = new Dictionary<string, AtomValue>();
            this.type = type;
            FillValues();
            return this;
        }

        public void FillValues()
        {
            switch (type)
            {
                case AtomType.SPEED:
                case AtomType.HEALTH:
                case AtomType.AGILITY:
                case AtomType.SMALLNESS:
                    values.Add("value", new AtomValue());
                    break;
                case AtomType.ARMOR:
                    values.Add("size", new AtomValue());
                    values.Add("thickness", new AtomValue());
                    values.Add("health", new AtomValue());
                    values.Add("armor", new AtomValue());
                    values.Add("direction", new AtomValue());
                    values.Add("distance", new AtomValue());
                    break;
                case AtomType.GUN:
                    values.Add("spread", new AtomValue());
                    values.Add("fspeed", new AtomValue());
                    values.Add("pspeed", new AtomValue());
                    values.Add("damage", new AtomValue());
                    values.Add("clipsize", new AtomValue());
                    values.Add("reloadspeed", new AtomValue());
                    break;
                case AtomType.LASER:
                    values.Add("duration", new AtomValue());
                    values.Add("cooldown", new AtomValue());
                    values.Add("damage", new AtomValue());
                    values.Add("arcstart", new AtomValue());
                    values.Add("arcend", new AtomValue());
                    values.Add("r", new AtomValue());
                    values.Add("g", new AtomValue());
                    values.Add("b", new AtomValue());
                    break;
                case AtomType.ROCKET:
                    values.Add("fspeed", new AtomValue());
                    values.Add("maxspeed", new AtomValue());
                    values.Add("agility", new AtomValue());
                    values.Add("damage", new AtomValue());
                    values.Add("direction", new AtomValue());
                    values.Add("pollution", new AtomValue());
                    break;
                case AtomType.COLOR:
                    values.Add("r", new AtomValue());
                    values.Add("g", new AtomValue());
                    values.Add("b", new AtomValue());
                    break;
                case AtomType.AGGRO:
                    values.Add("value", new AtomValue());
                    break;
                case AtomType.JINK:
                    values.Add("value", new AtomValue());
                    break;
                case AtomType.FREQ:
                    values.Add("value", new AtomValue());
                    break;
                case AtomType.AMP:
                    values.Add("value", new AtomValue());
                    break;
            }
        }

        /// <summary>
        /// Add a new value to this atom
        /// </summary>
        /// <param name="val"></param>
        public void AddValue(string key, AtomValue value)
        {
            values.Add(key, value);
        }

        /// <summary>
        /// Mutate this atom
        /// </summary>
        /// <returns></returns>
        public Atom Mutate()
        {
            Atom ret = new Atom();
            ret.values = new Dictionary<string, AtomValue>();
            ret.type = type;
            foreach (string key in values.Keys)
            {
                if (Game1.random.NextDouble() <= mutationRate)
                    ret.AddValue(key, new AtomValue());
                else
                    ret.AddValue(key, values[key]);
            }
            return ret;
        }

        public void Write(BinaryWriter writer)
        {
            writer.Write((int)type);
            writer.Write(values.Count);
            foreach (KeyValuePair<string, AtomValue> kv in values)
            {
                writer.Write(kv.Key);
                writer.Write(kv.Value);
            }
        }

        public static Atom Read(BinaryReader reader)
        {
            Atom ret = new Atom();
            ret.values = new Dictionary<string, AtomValue>();
            ret.type = (AtomType)reader.ReadInt32();
            int valcount = reader.ReadInt32();
            for (int c = 0; c < valcount; c++)
                ret.AddValue(reader.ReadString(), (int)reader.ReadInt32());
            return ret;
        }
    }
}
