﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.IO;

namespace Primeval
{
    static class MainMenu
    {
        private static Texture2D header;
        private static Texture2D letter;
        private static Texture2D menu;
        private static Texture2D selector;

        private static Vector2[] menuPositions;

        private static string[] exports;

        private static int cmode;
        private static int nextMode;
        private static int mode
        {
            get { return cmode; }
            set
            {
                nextMode = value;
                menuOption = 0;
            }
        }

        private static int firstSelectedExport = -1;

        private static int menuOption;

        private static bool selectPressed = true;
        private static bool dirPressed = true;

        private static float transitionPosition;

        private static Vector2 oldMousePos;

        private static Vector2 exportsTop = new Vector2(100, 220);
        private static float exportsLH { get { return Game1.exportsFont.MeasureString("Hj").Y + 5; } }

        static MainMenu()
        {
            menuPositions = new Vector2[4];
            menuPositions[0] = new Vector2(80, 344);
            menuPositions[1] = new Vector2(80, 419);
            menuPositions[2] = new Vector2(80, 494);
            menuPositions[3] = new Vector2(80, 569);
        }

        public static void LoadContent()
        {
            header = Game1.content.Load<Texture2D>("header");
            letter = Game1.content.Load<Texture2D>("letter");
            menu = Game1.content.Load<Texture2D>("menu");
            selector = Game1.content.Load<Texture2D>("selector");
        }

        private static int maxMenu { get { return (mode == 1 ? menuPositions.Length : exports.Length + 1); } }

        public static void Update(GameTime gameTime)
        {
            KeyboardState kstate = Keyboard.GetState();
            GamePadState gstate = GamePad.GetState(PlayerIndex.One);
            MouseState mstate = Mouse.GetState();

            if (nextMode != mode)
            {
                transitionPosition -= (float)gameTime.ElapsedGameTime.TotalSeconds / 0.05f;
                if (transitionPosition <= 0)
                {
                    transitionPosition = 0;
                    cmode = nextMode;
                }
            }
            else
            {
                if (transitionPosition < 1)
                {
                    transitionPosition += (float)gameTime.ElapsedGameTime.TotalSeconds / 0.05f;
                    if (transitionPosition > 1) transitionPosition = 1;
                }
            }

            if (mode == 1 || mode == 2)
            {
                //Mouse control
                if (oldMousePos != new Vector2(mstate.X, mstate.Y)
                    && mstate.X > menuPositions[0].X
                    && mstate.X < menuPositions[0].X + selector.Width)
                {
                    int option = menuOption;
                    if (mode == 1)
                        option = (int)(mstate.Y - menuPositions[0].Y + 39) / 77;
                    else if (mode == 2)
                        option = (int)(mstate.Y - exportsTop.Y) / (int)exportsLH;
                    if (option >= 0 && option < maxMenu)
                    {
                        if (option != menuOption) Game1.soundbank.PlayCue("MenuClick");
                        menuOption = option;
                    }
                }

                //Button control
                if (kstate.IsKeyDown(Keys.Up) || gstate.IsButtonDown(Buttons.LeftThumbstickUp))
                {
                    if (!dirPressed)
                    {
                        dirPressed = true;
                        menuOption--;
                        if (menuOption < 0)
                            menuOption = 0;
                        else
                            Game1.soundbank.PlayCue("MenuClick");
                    }
                }
                else if (kstate.IsKeyDown(Keys.Down) || gstate.IsButtonDown(Buttons.LeftThumbstickDown))
                {
                    if (!dirPressed)
                    {
                        dirPressed = true;
                        menuOption++;
                        if (menuOption >= maxMenu)
                            menuOption = maxMenu - 1;
                        else
                            Game1.soundbank.PlayCue("MenuClick");
                    }
                }
                else
                    dirPressed = false;
            }

            if (kstate.IsKeyDown(Keys.Enter) ||
                gstate.IsButtonDown(Buttons.Start) ||
                gstate.IsButtonDown(Buttons.A) ||
                mstate.LeftButton == ButtonState.Pressed)
            {
                if (!selectPressed)
                {
                    selectPressed = true;
                    if (mode == 0)
                        mode = 1;
                    else if (mode == 1)
                    {
                        if (menuOption == 0)
                            mode = 0;
                        else if (menuOption == 1)
                        {
                            Game1.transitionPosition = 1;
                            Game1.currentState = MenuState.GAME;
                            Game1.Reset();
                        }
                        else if (menuOption == 2)
                        {
                            mode = 2;
                            exports = Directory.GetFiles("Exports");
                        }
                        else if (menuOption == 3)
                            Environment.Exit(0);
                    }
                    else if (mode == 2)
                    {
                        if (menuOption > 0)
                        {
                            if (firstSelectedExport > 0)
                            {
                                Game1.currentState = MenuState.GAME;
                                Game1.Reset();
                                GenePool.AddSeedBot(Genome.Read(exports[firstSelectedExport - 1]));
                                GenePool.AddSeedBot(Genome.Read(exports[menuOption - 1]));
                                Bot.Reset();
                                GameState.currentState = State.BREED;
                                GameState.newState = false;
                                firstSelectedExport = -1;
                                mode = 1;
                            }
                            else
                                firstSelectedExport = menuOption;
                        }
                        else
                            mode = 1;
                    }
                }
            }
            else
                selectPressed = false;

            oldMousePos = new Vector2(mstate.X, mstate.Y);
        }

        public static void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(
                header,
                Vector2.Zero,
                Color.White);

            if (mode == 0)
            {
                spriteBatch.Draw(
                    letter,
                    Vector2.Zero,
                    new Color(1, 1, 1, transitionPosition));
            }
            else if (mode == 1)
            {
                spriteBatch.Draw(
                    menu,
                    Vector2.Zero,
                    new Color(1, 1, 1, transitionPosition));
                spriteBatch.Draw(
                    selector,
                    menuPositions[menuOption],
                    new Color(1, 1, 1, transitionPosition));
            }
            else if (mode == 2)
            {
                Vector2 pos = exportsTop;
                float lineHeight = exportsLH;

                spriteBatch.Draw(
                    selector,
                    pos + new Vector2(0, lineHeight * menuOption + 18),
                    new Color(1, 1, 1, transitionPosition));

                if (firstSelectedExport > 0)
                {
                    spriteBatch.Draw(
                        selector,
                        pos + new Vector2(0, lineHeight * firstSelectedExport + 18),
                        new Color(1, 1, 1, transitionPosition));
                }

                pos.X = 295;

                spriteBatch.DrawString(
                    Game1.exportsFont,
                    "Back",
                    pos - new Vector2(Game1.exportsFont.MeasureString("Back").X / 2, 0),
                    Color.LightGray);
                pos.Y += lineHeight;
                if (exports.Length > 0)
                {
                    for (int c = 0; c < exports.Length; c++)
                    {
                        string str = Path.GetFileNameWithoutExtension(exports[c]);
                        spriteBatch.DrawString(
                            Game1.exportsFont,
                            str,
                            pos - new Vector2(Game1.exportsFont.MeasureString(str).X / 2, 0),
                            Color.LightGray);
                        pos.Y += lineHeight;
                    }
                }
                else
                {
                    spriteBatch.DrawString(
                        Game1.exportsFont,
                        "No products.\nProducts are saved in the 'Exports' folder.",
                        pos + new Vector2(-200, 50),
                        Color.LightGray);
                }
            }
        }
    }
}
