﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;

namespace Primeval
{
    /// <summary>
    /// This class represents a guided rocket on the screen
    /// </summary>
    class Rocket : Projectile
    {
        private float maxVelocity;

        private float acceleration;

        protected override float radius { get { return launcherRadius; } }

        private float launcherRadius;

        private static Texture2D rocketshadow;

        private TimeSpan totalSmokeSpawnTime;
        private TimeSpan smokeSpawnTime;

        private Cue cue;

        public Rocket(float velocity, float damage, float radius, Vector2 direction,
            Bot owner, float acceleration, int polrate)
            : base(velocity, damage, direction, owner)
        {
            launcherRadius = radius;
            maxVelocity = velocity;
            totalSmokeSpawnTime = TimeSpan.FromMilliseconds(polrate);
            this.acceleration = acceleration;
            //cue = Game1.soundbank.GetCue("Rocket");
            //cue.Play();
        }

        public static new void LoadContent()
        {
            rocketshadow = Game1.content.Load<Texture2D>("rocketshadow");
        }

        public override void Update(GameTime gameTime)
        {
            //Accelerate
            if (!owner.Die)
            {
                velocity += Vector2.Normalize(Game1.player.position - position)
                    * acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                if (velocity.Length() > maxVelocity) velocity = Vector2.Normalize(velocity) * maxVelocity;
            }

            //Spawn smoke particles
            smokeSpawnTime -= gameTime.ElapsedGameTime;
            if (smokeSpawnTime <= TimeSpan.Zero)
            {
                smokeSpawnTime = totalSmokeSpawnTime;
                new Particle(ParticleType.SMOKE, position, TimeSpan.FromSeconds(1));
            }

            //Update sound
            //TODO:

            base.Update(gameTime);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(
                rocketshadow,
                Game1.TransformToDraw(position) + new Vector2(0, 30),
                null,
                Color.White,
                (float)(Math.Atan2(velocity.Y, velocity.X) + Math.PI / 2),
                new Vector2(rocketshadow.Width, rocketshadow.Height) / 2,
                radius * 2 / rocket.Width,
                SpriteEffects.None,
                1f);
            spriteBatch.Draw(
                rocket,
                Game1.TransformToDraw(position),
                null,
                Color.White,
                (float)(Math.Atan2(velocity.Y, velocity.X) + Math.PI / 2),
                new Vector2(rocket.Width, rocket.Height) / 2,
                radius * 2 / rocket.Width,
                SpriteEffects.None,
                1f);
        }

        protected override void Die(Vector2 normal)
        {
            //Spawn explosion particles
            for (int c = 0; c < 8; c++)
            {
                double ang = Math.Atan2(normal.Y, normal.X);
                Vector2 velocity = new Vector2(
                    (float)Math.Cos(ang + Game1.random.NextDouble() * 0.6),
                    (float)Math.Sin(ang + Game1.random.NextDouble() * 0.6)) * (Game1.random.Next(150) + 20);
                new Particle(
                    ParticleType.FLAME, position,
                    velocity,
                    Vector2.Zero,
                    TimeSpan.FromMilliseconds(500),
                    false);
                new Particle(
                    ParticleType.SMOKE, position,
                    velocity,
                    Vector2.Zero,
                    TimeSpan.FromMilliseconds(1000),
                    true);
            }

            //Stop sound
            //cue.Stop(AudioStopOptions.AsAuthored);
            //cue.Dispose();
            //cue = null;

            base.Die(normal);
        }
    }
}
